# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class HousingOffersController < Hermes::Api::Dmo::ApplicationController
        hermes_errors 'housing_channel_error'

        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s
        set_instance_for :eligible_requests, :with_validated_applications_for, :history
        skip_before_action :verify_authenticity_token, only: [:with_exchange_token]
        skip_before_action :set_user, only: [:with_exchange_token]
        skip_before_action :set_instance, only: [:with_exchange_token]

        VALIDATED_APPLICATION_STATUS = 'TRANSMIS BAILLEUR'

        MANAGEMENT_ROLES = [
          'Administrator',
          'Manager',
          'superadministrateur',
          'entreprise',
          'bailleur',
          'collaborateur offre logement'
        ].freeze

        class << self
          def mutate_params
            {
              accept_low_occupation: { type: 'Boolean'},
              address: { type: 'String'},
              address_can_be_shared: { type: 'String'},
              address_complements: { type: 'String'},
              address_extended: { type: 'String'},
              administration_area: { type: 'String'},
              administrator: { type: 'String'},
              age: { type: 'String'},
              agency_id: { type: 'String'},
              agency_name: { type: 'String'},
              already_inhabited: { type: 'Boolean'},
              already_rent: { type: 'Boolean'},
              als_siren: { type: 'String'},
              application_following_contact: { type: 'String'},
              auto_destroy_next_day: { type: 'Boolean'},
              availability_date: { type: 'DateTime'},
              bathrooms: { type: 'Integer'},
              bedrooms: { type: 'Integer'},
              building_type: { type: 'String'},
              cellars: { type: 'Integer'},
              commercialized: { type: 'Boolean'},
              communicate_exact_address: { type: 'Boolean'},
              condo: { type: 'Boolean'},
              construction_type: { type: 'String'},
              construction_year: { type: 'Date'},
              country: { type: 'String'},
              dalo: { type: 'Boolean'},
              date_publication_end: { type: 'DateTime'},
              date_publication_start: { type: 'DateTime'},
              delai_reponse: { type: 'Integer'},
              description: { type: 'String'},
              details: { type: 'String'},
              disability_accesses: { type: 'Integer'},
              district: { type: 'String'},
              dpe_conso: { type: 'String'},
              dpe_ges: { type: 'String'},
              duplex: { type: 'Boolean'},
              effort_rate: { type: 'Float'},
              elevators: { type: 'Integer'},
              entities_siret: { type: 'Array'},
              entry_date: { type: 'DateTime'},
              exclusivity_end_date: { type: 'DateTime'},
              expiry_days: { type: 'Integer'},
              external_ref: { type: 'String'},
              first_quartile: { type: 'String'},
              floor: { type: 'Integer'},
              floors: { type: 'Integer'},
              furnished_lodging: { type: 'Boolean'},
              gardens: { type: 'Integer'},
              green_area: { type: 'Boolean'},
              guarantee_deposit: { type: 'Float'},
              has_gardian: { type: 'Boolean'},
              heating_type: { type: 'String'},
              hot_water_distribution: { type: 'String'},
              individual_heating: { type: 'Boolean'},
              insee_code: { type: 'String'},
              is_from_form: { type: 'Boolean'},
              is_public: { type: 'Boolean'},
              kind: { type: 'String'},
              kitchen_type: { type: 'String'},
              lessor_code: { type: 'String'},
              lessor_contact_name: { type: 'String'},
              lessor_convention_reference: { type: 'String'},
              lessor_email: { type: 'String'},
              lessor_orientations: { type: 'Array'},
              lessor_phone_number: { type: 'String'},
              lessor_siren: { type: 'String'},
              lessor_status: { type: 'String'},
              latitude: { type: 'Float'},
              longitude: { type: 'Float'},
              mad_als_date: { type: 'DateTime'},
              min_effort_rate: { type: 'Float'},
              min_effort_treshold: { type: 'Float'},
              mobility: { type: 'Boolean'},
              mutation: { type: 'Boolean'},
              nb_of_eligible_employees: { type: 'Integer'},
              nb_of_requests_from_employees: { type: 'Integer'},
              new_offer: { type: 'Boolean'},
              number_of_companies_30_minutes_around: { type: 'Integer'},
              offer_status: { type: 'String'},
              operation_name: { type: 'String'},
              ordered_picture_ids: { type: 'Array'},
              other_quartiles: { type: 'String'},
              outside: { type: 'String'},
              parking_charges: { type: 'Float'},
              parking_type: { type: 'String'},
              picture_urls: { type: 'Array'},
              pmr: { type: 'Boolean'},
              postal_code: { type: 'String'},
              potential_request_name_county: { type: 'String'},
              potential_request_name_town: { type: 'String'},
              priority_public: { type: 'Boolean'},
              publication_end_date: { type: 'DateTime'},
              qpv: { type: 'Boolean'},
              reference_mad: { type: 'String'},
              rent_amount: { type: 'Float'},
              rent_with_charges: { type: 'Float'},
              rental_charges: { type: 'Float'},
              requalification: { type: 'Boolean'},
              reservation_agreement_ref: { type: 'String'},
              reserving_siren: { type: 'String'},
              residence_code: { type: 'String'},
              residence_title: { type: 'String'},
              rooms: { type: 'Integer'},
              rpls_id: { type: 'String'},
              single_parent: { type: 'Boolean'},
              subsidiary_company: { type: 'String'},
              surface: { type: 'Integer'},
              tense_area: { type: 'Boolean'},
              terraces_or_balconies: { type: 'Integer'},
              test_created_at: { type: 'DateTime'},
              threshold_subtype: { type: 'String'},
              threshold_type: { type: 'String'},
              total_floors: { type: 'String'},
              type: { type: 'String'},
              typology: { type: 'String'},
              virtual_visit: { type: 'Boolean'},
            }
          end 
      
          def acceptable_params
            {
              active_reservations: { type: 'Array'},
              async_eligible_bordering_nb: { type: 'Integer'},
              async_eligible_department_nb: { type: 'Integer'},
              async_eligible_other_from_department_nb: { type: 'Integer'},
              async_eligible_requests_nb: { type: 'Integer'},
              async_eligible_seeked_bordering_nb: { type: 'Integer'},
              async_eligible_seeked_nb: { type: 'Integer'},
              ceil_category_1: { type: 'Float'},
              ceil_category_10: { type: 'Float'},
              ceil_category_11: { type: 'Float'},
              ceil_category_12: { type: 'Float'},
              ceil_category_2: { type: 'Float'},
              ceil_category_3: { type: 'Float'},
              ceil_category_4: { type: 'Float'},
              ceil_category_5: { type: 'Float'},
              ceil_category_6: { type: 'Float'},
              ceil_category_7: { type: 'Float'},
              ceil_category_8: { type: 'Float'},
              ceil_category_9: { type: 'Float'},
              country: { type: 'String'},
              department: { type: 'String'},
              designation_date_end: { type: 'DateTime'},
              floor_category_1: { type: 'Float'},
              floor_category_10: { type: 'Float'},
              floor_category_11: { type: 'Float'},
              floor_category_12: { type: 'Float'},
              floor_category_2: { type: 'Float'},
              floor_category_3: { type: 'Float'},
              floor_category_4: { type: 'Float'},
              floor_category_5: { type: 'Float'},
              floor_category_6: { type: 'Float'},
              floor_category_7: { type: 'Float'},
              floor_category_8: { type: 'Float'},
              floor_category_9: { type: 'Float'},
              has_cellar: { type: 'Boolean'},
              has_disability_access: { type: 'Boolean'},
              has_elevator: { type: 'Boolean'},
              has_garden: { type: 'Boolean'},
              is_mad_updated_at: { type: 'DateTime'},
              is_mad: { type: 'Boolean'},
              last_update_by_lessor: { type: 'DateTime'},
              lessor_id: { type: 'String'},
              lessor_name: { type: 'String'},
              lessor_status_updated_at: { type: 'DateTime'},
              min_revenues: { type: 'Float'},
              offer_status_updated_at: { type: 'DateTime'},
              reservation_holder_id: { type: 'String'},
              reserved: { type: 'Boolean'},
              reserving_business_name: { type: 'String'},
              total_charges: { type: 'Float'},
              virtual_visit: { type: 'Boolean'},
            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $gt $gte $lt $lte $regex]
          end

          def regex_params
            {}
          end 
        end

        LESSOR_FORBIDDEN_PARAMS = %i[
          als_siren
          dalo
          status
          status_updated_at
        ]

        def mutate_params
          current_user && current_lessor ? MUTATE_PARAMS - LESSOR_FORBIDDEN_PARAMS : MUTATE_PARAMS + LESSOR_FORBIDDEN_PARAMS
        end 

        def handle_offers_eligibility
          applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)
          hr = applicant&.first_active_request
          @query_precision ||= {}

          eligibility_query = if %w[Administrator bailleur].include? current_user_roles
            {}
          else
            if hr&.all_eligibility_query
              hr.all_eligibility_query.merge({published: true})
            else           
              {is_public: true}
            end
          end

          @query_precision = { :$and =>[@query_precision,eligibility_query]}
        end


        def search
          index
        end

        def index_parameters_settings
          acceptable_params *self.class.acceptable_params.keys - [:longitude, :latitude]
          valid_operators *self.class.valid_operators
        end

        def mutate_valid_params
          valid_params = super
          
          if @instance &&
            [@instance.ordered_picture_ids, @instance.picture_urls].any? { |list| list&.any? } &&
            [valid_params[:ordered_picture_ids], valid_params[:picture_urls]].compact.all?(&:empty?)
            valid_params.delete(:ordered_picture_ids)
            valid_params.delete(:picture_urls)
          end
        
          valid_params
         end 

        def create
          sanitize_string_as_integer
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
          @model.current_user_id = @user.id
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          Rails.logger.debug "TEST before #{user_id}"
          user_id = param_user if @user.has_role?('Administrator') && param_user
          Rails.logger.debug "TEST #{mutate_valid_params} after #{user_id}"
          sanitize_status
          @instance = @model.new(mutate_valid_params.merge(user_id: user_id))

          lessor_criteria = @instance.lessor_code == "000" ? { "$in": [nil, "000"]} : @instance.lessor_code 
          found_instance = Hermes::HousingOffer.order_by(created_at: :desc).find_by(
            external_ref: @instance.external_ref, 
            lessor_siren: @instance.lessor_siren,
            lessor_code: lessor_criteria,
          )

          if !found_instance || (found_instance && found_instance.need_to_duplicate_offer)
 
            @instance.with_lock(reload: true) do
              @instance.last_update_by_lessor = DateTime.current
              return respond_with_errors(@instance) unless @instance.save
              render(jsonapi: @instance, serializer: @serializer, status: :created)
            end
          else
         
            @instance = found_instance
            @instance.with_lock(reload: true) do
              return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && @instance.writable_by?(@user)
              valid_params = mutate_valid_params
              valid_params[:user_id] = params['data']['attributes'][:user_id] if params['data']['attributes'][:user_id] && @user.has_role?('Administrator')
              valid_params.delete(:lessor_status)
              valid_params.delete(:offer_status)
              valid_params.delete(:reserving_business_name)
              valid_params.delete(:details) unless found_instance.details.blank?

              return render_json_error(:unprocessable_action, :no_valid_parameters) unless valid_params.present?
              Rails.logger.debug "TEST valid_params are #{valid_params}"
              @instance.last_update_by_lessor = DateTime.current
              return respond_with_errors(@instance) unless @instance.update(valid_params) && @instance.save

              render(jsonapi: @instance, serializer: @serializer)
            end

          end
        end

        def aggregation_with_eligibility(other_query = nil)
          aggregation = if ["Administrator", "bailleur", "entreprise", "superadministrateur", "cre", "rdsl", "collaborateur offre logement"].any? {|role| current_user_roles.include?(role)}
            index_aggregation
          else
            handle_offers_eligibility
            @query_precision = { :$or =>[@query_precision, other_query]} if other_query
            index_aggregation(no_acl: true)
          end
        end 

        def index
          index_parameters_settings
          aggregation = aggregation_with_eligibility
          applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          if params[:postal_codes] && params[:postal_codes].is_a?(Array)
            aggregation = aggregation.search_by_postal_codes(params[:postal_codes])
          end

          if params[:longitude] && params[:latitude]
            distance = params[:distance] || 20
            unit = params[:unit] || :km
            ids = @model.near([params[:longitude], params[:latitude]], distance, unit: unit).map { |ho_v| ho_v['_id'] }
            aggregation = aggregation.where(id: { :$in => ids })
          end

          @options = { 
            eligible_requests_nb: params['options']&.include?('eligible_requests_nb'),
            possible_eligible_requests_nb: params['options']&.include?('possible_eligible_requests_nb')
          }

          paginate(
            aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def show
          @options = {
            include: %i[user hermes-lessor entities threshold],
            fields: {'hermes-lessor'.to_sym => ['title']},
            eligible_count: params['options']&.include?('eligible_count'),
            eligible_requests_nb: params['options']&.include?('eligible_requests_nb'),
            possible_eligible_requests_nb: params['options']&.include?('possible_eligible_requests_nb')
          }
          current_user_active_application = Hermes::HousingApplication.find_by(is_active: true, user: current_user, housing_offer_id: params[:id])

          base_query = if %w[Administrator bailleur entreprise superadministrateur cre rdsl].any? {|role| current_user_roles.include?(role)}
                         @model.readable_by(@user)
                       elsif is_collaborator_offer || current_user_active_application
                         @model    
                       else
                         handle_offers_eligibility if @user
                         @model
                       end
          @instance = base_query.find_by(id: params[:id], **@query_precision) if params[:id]
          
          super
        end

        def update
          @model.current_user_id = @user.id
          sanitize_status
          sanitize_string_as_integer
          # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

          return render_json_error(:not_found, :object_not_found) unless @instance

          if !is_admin &&
            params['data']['attributes']['offer_status'] == "LOGEMENT PUBLIE" && 
            @instance.offer_status != "LOGEMENT PUBLIE"
            return render_json_error(:unauthorized, :not_right_to_publish) 
          end 

          @instance.with_lock(reload: true) do
            super
          end

        end

        def is_collaborator_offer
          current_user.auth_roles&.include?('collaborateur offre logement')
        end

        def cotations_order_params
          o_params = order_params.to_h
          if o_params["cotations"] && @instance&.insee_code
            o_params["cotations.#{@instance.insee_code}"] = o_params['cotations']
            o_params.delete("cotations")
          end 
          o_params
        end 

        def all_eligible_requests
          return Hermes::HousingRequest.where(id: "") unless @instance.eligibility_query_base
          
          other_query_params = { :$and => [ :$and => [build_query(Hermes::HousingRequest)]] }

          requests = Hermes::HousingRequest.where(@instance.all_eligibility_query).where(other_query_params).order_by(cotations_order_params)
        end


        def eligible_requests
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers? || @user.auth_roles&.include?('collaborateur offre logement')

          acceptable_params *Hermes::Api::Dmo::HousingRequestsController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingRequestsController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::HousingRequestsController.valid_operators

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          other_query_params = { :$and => [ :$and => [build_query(Hermes::HousingRequest)]] }
          eligibility_type = params[:eligibility_type] || 'department'
          aggregation = @instance.eligibles_requests(eligibility_type).where(other_query_params).order_by(cotations_order_params)
          entities = nil

          if params[:reserving_siren].present?
            entities = Hermes::ReservationHolder.find_entities_from_siren(params[:reserving_siren])
            aggregation = aggregation.where(:entity_id.in => entities) unless entities.nil?
          end 

          other_count = {}
          Hermes::HousingOffer.eligibility_types.each do |type|
            if params['options']&.include?("#{type}_count")
              eligibles_requests =  @instance.eligibles_requests(type).where(other_query_params)
              eligibles_requests = eligibles_requests.where(:entity_id.in => entities) unless entities.nil?
              other_count["#{type}_count"] = eligibles_requests.count(hint: Hermes::HousingOffer.find_hint(type))
            end 
          end 

          options[:other_count] = other_count unless other_count.empty?
          options[:hint] = Hermes::HousingOffer.find_hint(eligibility_type)

          paginate(
            aggregation,
            each_serializer: Hermes::HousingRequestSerializer,
            **options
          )
        end

        def is_eligible_request
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_acceptable, :missing_als_id) if params[:als_id].blank?
          return render_json_error(:not_found, :request_not_found) unless hr = Hermes::HousingRequest.find_by(als_id: params[:als_id])
      
          render json: { is_eligible: !!Hermes::HousingRequest.where(@instance.all_eligibility_query).find_by(id: hr.id.to_s) }
        end 

        def eligible_requests_rights_filtered
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unauthorized, :not_manager) unless can_manage_offers? || @user.auth_roles&.include?('collaborateur offre logement')

          acceptable_params *Hermes::Api::Dmo::HousingRequestsController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingRequestsController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::HousingRequestsController.valid_operators

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          entity_ids = current_user_entity_ids
          entity_ids = current_user_entity_ids & params[:entity_ids] if params[:entity_ids].present?

          ap all_eligible_requests
          paginate(
            all_eligible_requests.where(:$and => [:entity_id.in => entity_ids]),
            each_serializer: Hermes::HousingRequestSerializer,
            **options
          )
        end 


        def entity_statuses
          @instance = @model.find_by(id: params[:id]) if params[:id]
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:not_acceptable, :missing_entity_id_params) unless params[:entity_id]

          can_read_entity = Hermes::Entity.readable_by(current_user).find_by(id: params[:entity_id])
          return render_json_error(:unauthorized, :no_access_to_entity) unless can_read_entity

          aggregation = Hermes::Status.where(housing_offer_id: @instance.id, status_label: 'entity_status', performer_id: params[:entity_id])

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          paginate(
            aggregation,
            each_serializer: Hermes::StatusSerializer,
            **options
          )
        end

        def all_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance && can_manage_offers?
          
          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::HousingApplicationsController.valid_operators

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          aggregation = Hermes::HousingApplication.actives.where(housing_offer_id: @instance.id).where(build_query(Hermes::HousingApplication)).order(cotations_order_params)

          options[:lignt_serializer] = true 

          paginate(
            aggregation,
            each_serializer: Hermes::HousingApplicationSerializer,
            **options
          )
        end

        def all_applications_rights_filtered
          return render_json_error(:not_found, :instance_not_found) unless @instance && can_manage_offers?

          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::HousingApplicationsController.valid_operators

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          entity_ids = current_user_entity_ids
          entity_ids = current_user_entity_ids & params[:entity_ids] if params[:entity_ids].present?

          aggregation = @instance.find_housing_application_with_entity_id(entity_ids).where(build_query(Hermes::HousingApplication)).order(cotations_order_params)
          options[:lignt_serializer] = true 

          paginate(
            aggregation,
            each_serializer: Hermes::HousingApplicationSerializer,
            **options
          )
        end

        def paginate_applications(applications)
          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::HousingApplicationsController.valid_operators

          applications = applications.readable_by(@user).where(build_query(Hermes::HousingApplication))
                           .order(order_params)
                           .includes(housing_request: [:main_applicant])
          options = {}
          options[:join_housing_request] = true
          paginate(
            applications,
            each_serializer: HousingApplicationSerializer,
            **options
          )
        end

        def selected_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers? #todo : a vérifier lorsqu'on aura la gestion de droits spécifiée

          applications = Hermes::HousingApplication.selected_by_offer @instance.id
          paginate_applications(applications)
        end

        def can_manage_offers?
          (MANAGEMENT_ROLES & current_user_roles)&.length&.send('>', 0)
        end

        def render_json_error_not_active_selected(housing_application_ids, concerned_applications, housing_offer_id)
          errors = []
          diff = housing_application_ids.uniq - concerned_applications.pluck(:id).map(&:to_s)
          diff.each do |id|
            ha = Hermes::HousingApplication.find(id)
            if !ha
              errors.push("Candidature (#{id}) introuvable") 
            elsif !ha.is_active
              errors.push("Candidature (id : #{id} / als_id : #{ha.housing_request.als_id}) inactive, statut : #{ha.status}, nud : #{ha.housing_request.unique_numbers.join(',')}") 
            else
              errors.push("Candidature (id : #{id} / als_id : #{ha.housing_request.als_id}) non associée à cette offre (#{housing_offer_id}), statut : #{ha.status}, nud : #{ha.housing_request.unique_numbers.join(',')}") 
            end 
          end 

          render(json: errors, status: :unprocessable_entity)
        end 

        def post_selected_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :not_manager) unless can_manage_offers? #todo : a vérifier lorsqu'on aura la gestion de droits spécifiée

          housing_application_ids = params.dig(:data, :attributes, :housing_application_ids)
          return render_json_error(:unprocessable_entity, :no_valid_parameters) unless housing_application_ids

          concerned_applications = Hermes::HousingApplication.where(
            is_active: true,
            :id.in => housing_application_ids, 
            housing_offer_id: @instance.id
          )

          unless housing_application_ids.uniq.length == concerned_applications.count
            return render_json_error_not_active_selected(housing_application_ids, concerned_applications, @instance.id) 
          end 

          @instance.update(
            lessor_status: "EN ATTRIBUTION",
            offer_status: "EN COURS D'ATTRIBUTION"
          )

          concerned_applications.each do |ha| 
            ha.update(is_active: true, status: "TRANSMIS BAILLEUR")
          end

          Hermes::HousingApplication.where(
            is_active: true,
            housing_offer_id: @instance.id,
            :id.nin => housing_application_ids,
          ).each do |ha| 
            ha.update(is_active: false, status: "NON RETENUE")
          end

          options = {join_housing_request: true}
          paginate(
            concerned_applications,
            each_serializer: HousingApplicationSerializer,
            join_housing_request: true
          )
        end

        def from_param
          from = params['from']
          return render_json_error(:unprocessable_entity, :missing_from_parameter) unless from
          begin
            DateTime.parse(from)
          rescue
            return render_json_error(:unprocessable_entity, :malformed_from_parameter)
          end
        end

        def from_to_query
          from = from_param
          return from unless from.is_a?(DateTime)

          query = {status_updated_at: {:$gte => from}}
          to = DateTime.parse(params[:to]) rescue nil
          query[:status_updated_at][:$lte] = to if to.is_a? DateTime
          query
        end

        def with_validated_applications
          from_to = from_to_query
          return from_to unless from_to.is_a?(Hash)

          application_query = {
            status: VALIDATED_APPLICATION_STATUS,
            is_active: true
          }.merge(from_to)

          validated_applications = Hermes::HousingApplication.where(
            application_query
          )
          housing_offer_ids = validated_applications.pluck(:housing_offer_id)
                                .uniq
                                .compact
          @query_precision = {:id.in => housing_offer_ids}
          index_parameters_settings
          aggregation = index_aggregation
          options = {}

          unless ['false', false].include?(params[:pagination])
            paginate(
              aggregation,
              each_serializer: Hermes::HousingOfferSerializer,
              **options
            )
          else
            render(
              jsonapi: aggregation,
              each_serializer: Hermes::HousingOfferSerializer,
              **options
            )
          end
        end

        def with_validated_applications_for
          def include_relationships
            []
          end

          from_to = from_to_query
          return from_to unless from_to.is_a?(Hash)
          return render_json_error(:not_found, :object_not_found) unless @instance

          acceptable_params *Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingApplicationsController.regex_params.keys
          valid_operators *Hermes::Api::Dmo::HousingApplicationsController.valid_operators

          applications_query = {
            housing_offer_id: @instance.id,
            status: VALIDATED_APPLICATION_STATUS,
            is_active: true
          }.merge(from_to)
          @query_precision = applications_query
          aggregation = index_aggregation(target_model: Hermes::HousingApplication)
          options = {embedded_applicant: true}

          unless ['false', false].include?(params[:pagination])
            paginate(
              aggregation,
              each_serializer: Hermes::HousingApplicationSerializer,
              **options
            )
          else
            render(
              jsonapi: aggregation,
              each_serializer: Hermes::HousingApplicationSerializer,
              **options
            )
          end
        end

        def paginate_history(history)
          acceptable_params *%i[lessor_objective_id]
          history = history.where(build_query(Hermes::LessorObjectiveHistory)).order(order_params)
          paginate(
            history,
            each_serializer: Hermes::LessorObjectiveHistorySerializer
          )
        end

        def history
          if current_user_roles.include?('bailleur')
            @instance = @model.readable_by(@user).find_by(id: params[:id], **@query_precision) if params[:id]
          end

          return render_json_error(:not_found, :instance_not_found) unless @instance

          paginate_history(@instance.history)
        end

        def reference_exist
          return render_json_error(:unprocessable_entity, :missing_external_ref_parameter) unless params[:external_ref]

          offers = Hermes::HousingOffer.where({
            offer_status: { '$nin': Hermes::HousingOffer.end_of_life_statuses},
            external_ref: params[:external_ref]
          })

          paginate(
            offers,
            each_serializer: @serializer
          )
        end

        def add_housing_channel
          @instance ||= @model.find_by(id: params[:id])
          keys = params[:keys] || [params[:key]]
          # If an array of keys is given, delete all existing keys to remplace by array
          if @instance && params[:keys].present?
            @instance.set(housing_channel_keys: [], updated_at: DateTime.current)
          end
          update_housing_channel do
            keys.uniq.each do |key|
              if Hermes::HousingChannel.find_by(key: key)
                @instance.update_housing_channel_keys!(key)
              else
                render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
              end
            end
          end
        end

        def remove_housing_channel
          keys = params[:keys] || [params[:key]]
          update_housing_channel do
            keys.each do |key|
              if Hermes::HousingChannel.find_by(key: key)
                @instance.update_housing_channel_keys!(key, false)
              else
                render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
              end
            end
          end
        end

        def with_recount_eligible_requests_nb
          return render_json_error(:unprocessable_entity, :housing_offer_ids_params_is_missing) unless params[:housing_offer_ids].present?
          params[:housing_offer_ids] = [params[:housing_offer_ids]] if params[:housing_offer_ids].is_a?(String)
          entities = nil
      
          if params[:reserving_siren].present?
            entities = Hermes::ReservationHolder.find_entities_from_siren(params[:reserving_siren])
          end 
          
          @options = {
            eligible_requests_nb: true,
            possible_eligible_requests_nb: params['options']&.include?('possible_eligible_requests_nb'),
            entities_used_for_eligible_requests_nb: entities,
          }

          housing_offer_user_application_ids = Hermes::HousingApplication.where(user: current_user).pluck(:housing_offer_id)
          aggregation = aggregation_with_eligibility({id: {"$in": housing_offer_user_application_ids}}).where(:id.in => params[:housing_offer_ids])

          paginate(
            aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def with_recount_eligible_requests_nb_rights_filtered
          return render_json_error(:unprocessable_entity, :housing_offer_ids_params_is_missing) unless params[:housing_offer_ids].present?
          params[:housing_offer_ids] = [params[:housing_offer_ids]] if params[:housing_offer_ids].is_a?(String)

          entities = Hermes::UserRight.where(user: current_user).pluck(:entity_id).map(&:to_s) - [nil]
          entities = entities & params[:entity_ids] if params[:entity_ids].present?
          
          @options = {
            eligible_requests_nb: true,
            possible_eligible_requests_nb: params['options']&.include?('possible_eligible_requests_nb'),
            entities_used_for_eligible_requests_nb: entities,
            recount_applicated_nb: true,
          }

          paginate(
            Hermes::HousingOffer.where(:id.in => params[:housing_offer_ids]),
            each_serializer: @serializer,
            **@options
          )
        end 

        def resend_flux5_all_missing_offers
          date = params[:date].present? ? Date.parse(params[:date]) : Date.today - 7.days
          offers = Hermes::HousingOffer.where(is_mad: true, last_successful_xml_sent_date: nil, :offer_status.nin => Hermes::HousingOffer.end_of_life_statuses, :created_at.gte => date.beginning_of_day)
          offer_ids = offers.pluck(:id).map(&:to_s)
          offer_ids.each do |offer_id|
            ExecuteFlux5TaskJob.perform_later(offer_id)
          end 

          render(json: { success: true, offer_ids: offer_ids }, status: '200') 
        end 

        def resend_flux5
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render(json: { success: true }, status: '200') if   Rails.env.test? 

          res = ExecuteFlux5TaskJob.perform_now(@instance.id.to_s)

          return render_json_error(:unprocessable_entity, :error_on_flux5) if !res || res.code != 200

          render(json: { success: true }, status: '200') 
        end 
      

        def simulate_eligible_requests_count
          if [
            :threshold_type,
            :threshold_subtype,
            :rent_with_charges,
            :insee_code,
            :typology,
          ].any? { |p| !params[p].present?}
            return render_json_error(:unprocessable_entity, :missing_params)
          end 

          offer = Hermes::HousingOffer.new(
            offer_status: 'LOGEMENT PUBLIE',
            threshold_type: params[:threshold_type],
            threshold_subtype: params[:threshold_subtype],
            rent_with_charges: params[:rent_with_charges].to_f,
            insee_code: params[:insee_code],
            typology: params[:typology],
          )

          offer.update_threshold
          offer.threshold.set_offer_thresholds(offer)
          offer.update_min_revenues
          offer.update_department
          offer.update_housing_channel_keys

          offer.update_eligibility_query
          type = 'seeked_bordering'
          query = offer.eligibility_query(type)

          [
            :single_parent,
            :priority_public,
            :mutation,
          ].each do |att|
            query.delete(att)
          end 

          complete_eligible_requests_count = Hermes::HousingRequest.where(query.merge(is_active: true, request_status: 'COMPLETE')).count(hint: Hermes::HousingOffer.find_hint(type))
          possible_eligible_requests_count = Hermes::HousingRequest.where(query.merge(is_active: true, :request_status.nin => ['ANNULEE', 'SATISFAITE'])).count(hint: Hermes::HousingOffer.find_hint(type))

          render(json: { complete_eligible_requests_count: complete_eligible_requests_count, possible_eligible_requests_count: possible_eligible_requests_count }, status: '200') 
        end 

        def generate_exchange_token
          return render_json_error(:forbidden, :not_administrator) unless @user.has_role?('Administrator')
          return render_json_error(:unprocessable_entity, :missing_param) unless params[:reserving_siren].present? && params[:id].present?
          return render_json_error(:not_found, :housing_offer_not_found) unless @instance = @model.find_by(id: params[:id]) if params[:id]
          return render_json_error(:not_found, :reservation_holder_id_not_found) unless rh = Hermes::ReservationHolder.find_by(siren: params[:reserving_siren])
          return render_json_error(:forbidden, :reservation_holder_id_not_matching) unless @instance.reservation_holder_id.to_s == rh.id.to_s

          expiration_duration = 90

          payload = {
            data: {
              housing_offer_id: @instance.id.to_s,
              reserving_siren: params[:reserving_siren],
            },
            exp: Time.now.to_i + expiration_duration
          }

          token = JWT.encode payload, APP_CONFIG['token'], 'HS256'

          render :status => 200, :json => { token: token}
        end 

        def with_exchange_token
          @instance = @model.find_by(id: params[:id]) if params[:id]
          return render_json_error(:not_found, :housing_offer_not_found) unless @instance = @model.find_by(id: params[:id]) if params[:id]
          return render_json_error(:unauthorized, :token_not_found) unless bearer_token

          begin
            decoded_token = JWT.decode bearer_token, APP_CONFIG["token"], true, { algorithm: 'HS256' }
            data = decoded_token.first['data']
            return render_json_error(:forbidden, :housing_offer_id_not_matching) unless data["housing_offer_id"]&.to_s == @instance.id.to_s
            return render_json_error(:forbidden, :reservation_holder_id_not_matching) unless Hermes::ReservationHolder.find_by(siren: data["reserving_siren"])&.id.to_s == @instance.reservation_holder_id.to_s

            render(
              jsonapi: @instance,
              serializer: @serializer,
            )
          rescue JWT::ExpiredSignature
            return render_json_error(:unauthorized, :token_expired) 
          rescue => e
            return render_json_error(:unauthorized, :unable_to_decode_token)
          end
        end 

        private

        def update_housing_channel
          @instance ||= @model.find_by(id: params[:id])
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unauthorized, :not_allowed) unless can_manage_housing_channel?

          if (params[:keys].present? && params[:keys].is_a?(Array)) || params[:key].present?
            yield if block_given?

            render json: {
              success: true,
              channels: @instance.housing_channel_keys
            }, status: :ok
          else
            raise ActionController::ParameterMissing.new('key or keys')
          end

        rescue ActionController::ParameterMissing => e
          render_json_error(:not_acceptable, :parameter_missing, {message: e.message})
        rescue Hermes::Errors::HousingChannelError => e
          code = e.message.include?("a déjà été ajouté")  ? :conflict : :unprocessable_entity
          render_json_error(code, :housing_channel_error, {message: e.message})
        end

        def can_manage_housing_channel?
          if current_user_roles.include?('bailleur')
            lessor = Hermes::Lessor.find_by(id: @instance.lessor_id)
            lessor.present? && lessor.readable_by?(@user)
          else
            current_user_roles.include?('Administrator')
          end
        end

        def sanitize_status
          # only offer.lessor can setup Hermes::HousingOffer::ATTRIBUTION_STATUSES
          if Hermes::HousingOffer::ATTRIBUTION_STATUSES.include?(params[:data][:attributes][:offer_status])
            if current_lessor.nil? || current_lessor&.id&.to_s != mutate_valid_params[:lessor_id]
              @instance = @model.find(params[:id] || 0)
              params[:data][:attributes][:offer_status] = @instance&.offer_status_was
            end
          end
        end

        def sanitize_string_as_integer
          %w[elevator garden cellar disability_access].each do |extra_name|
            next unless params[:data][:attributes].key?(extra_name.pluralize.to_sym)

            val = params[:data][:attributes][extra_name.pluralize.to_sym] 
            if val.nil? || val.class == String
              params[:data][:attributes][extra_name.pluralize.to_sym] = ['1', 'O'].include?(val) ? 1 : val.to_i
            end                    
          end
        end 

        def log_skipable?
          # skip after_filter declared in core
          %i[index].include? action_name.to_sym
        end

        private 

        def current_user_entity_ids
          Hermes::UserRight.where(user: current_user).pluck(:entity_id).map(&:to_s) - [nil]
        end 
      end
    end
  end
end
