module Hermes
  module Api
    module Dmo
      class PicturesController < Hermes::Api::Dmo::ApplicationController
        before_action :check_upload_file, only: [:create, :add_picture_to_offer]
        
        def create
          return render_json_error :unauthorized, :cannot_upload_free_picture unless user_can_upload_free_picture?
          create_picture
        end

        def add_picture_to_offer
          return render_json_error :not_found, :housing_offer_not_found unless @housing_offer = Hermes::HousingOffer.find_by(id: params[:id])
          return render_json_error :unauthorized, :cannot_upload_to_offer unless user_can_upload_to_offer?
          create_picture
        end 

        def delete_picture_from_offer
          return render_json_error :not_found, :housing_offer_not_found unless @housing_offer = Hermes::HousingOffer.find_by(id: params[:id])
          @instance = Hermes::Picture.find_by(id: params[:picture_id])
          destroy
        end 
      
        private 

        def create_picture
          @instance = Hermes::Picture.new(user_id:  @user.id, writers: [@user.id], housing_offer_id: @housing_offer&.id)
          blob = @file.read
          @instance.set_blob(blob)

          return respond_with_errors(@instance) unless @instance.save
          render(jsonapi: @instance, serializer: Hermes::PictureSerializer, status: :created)
        end 

        def set_instance
          @instance = Hermes::Picture.find_by(id: params[:id])
        end

        def user_can_upload_free_picture?
          auth_roles_can_upload = %w[
            rdsl
            lessor
            collaborateur offer logement
          ]

          is_admin_or_super_admin || auth_roles_can_upload.any? {|role| current_user_roles.include?(role)}
        end 

        def user_can_upload_to_offer?
          is_admin_or_super_admin || @housing_offer&.readable_by?(@user)
        end 

        def user_can_destroy?
          is_admin_or_super_admin || @instance&.housing_offer&.readable_by?(@user) || @instance&.writable_by?(@user) 
        end 
        
        def check_upload_file
          return render_json_error :unprocessable_entity, :no_file_in_the_request unless @file = params[:file]
          
          begin 
            Magick::Image.ping(@file.path).first.nil?
          rescue
            return render_json_error :unsupported_media_type, :not_an_image 
          end
        end 
      end
    end
  end
end