module NotificationsCenter
  class HousingApplication < Base
    KEYS = NOTIFICATION_KEYS[:housing_application]
    def initialize(*arg)
      super
      @housing_application = @instance.notifiable
      @housing_offer = @housing_application.housing_offer
      @lessor = @housing_offer.lessor
      @name = 'Offer name'
      @number = @housing_offer.reference
      @city = @housing_offer.city
      @size = @housing_offer.surface
    end

    def title
      case @key
        when KEYS[:lessor_withdrawal]
          "#{NAME} - Information sur votre candidature"
        when KEYS[:lessor_accepted]
          "#{NAME} - Information sur votre candidature"
        when KEYS[:lessor_transferred]
          "#{NAME} - Dossier transmis au bailleur"
        when KEYS[:lessor_cancelled]
          "#{NAME} - Information sur votre candidature"
        when KEYS[:lessor_denied]
          "#{NAME} - Information sur votre candidature"
        when KEYS[:cal_denied]
          "#{NAME} - Information sur votre candidature"
        when KEYS[:pending], KEYS[:subscribed], KEYS[:applicant_cancelled], KEYS[:applicant_cancelled_after_visit], KEYS[:applicant_cancelled]
          # front only
          ""
        else
          ''
      end
    end

    def body
      case @key
        when KEYS[:lessor_withdrawal]
          "
            Nous vous informons que le logement situé #{@housing_offer.full_address}, sur lequel vous avez candidaté, n'est plus disponible. Le bailleur a annulé la mise à disposition de ce logement, nous ne pouvons donc plus vous le proposer.\n
            Nous vous invitons dès maintenant à vous rendre de nouveau sur votre espace, afin de postuler sur une autre offre de logement. 
          "
        when KEYS[:lessor_accepted]
          "
            Après étude de votre dossier par le bailleur sur le logement N°#{@number}, nous avons le plaisir de vous informer que celui-ci a retenu votre candidature.\n
            Le bailleur reviendra vers vous pour vous indiquer la marche à suivre : signature du bail, état des lieux et remise des clefs.\n
            Nous vous précisons que votre demande de logement sera clôturée dès que vous aurez signé votre bail.\n
            Merci pour votre confiance.
            <br/>
            Nous vous informons que des aides Action Logement permettent de faciliter votre installation, renseignez-vous :\n
            <ul>
              <li>une aide financière de 1 000 euros afin d'assurer votre déménagement et l'achat de vos meubles : tester mon éligibilité.</li>
              <li>un prêt à taux 0% pour vous permettre de payer votre caution et la rembourser sur plusieurs mois : Tester mon éligibilité.</li>
            </ul>
            N'hésitez pas à nous contacter pour avoir plus d'informations sur nos services et faire un diagnostic avec l'un de nos conseillers. 
          "
        when KEYS[:lessor_transferred]
          "Après étude de votre candidature sur le logement N°#{@number} nous vous informons que votre dossier a été retenu. Il a été envoyé au bailleur pour étude."
        when KEYS[:lessor_cancelled]
          "
            Nous vous informons que le logement situé #{@housing_offer.full_address}, sur lequel vous avez candidaté, n'est plus disponible. Le bailleur a annulé la mise à disposition de ce logement, nous ne pouvons donc plus vous le proposer.
            Nous vous invitons dès maintenant à vous rendre de nouveau sur votre espace, afin de postuler sur une autre offre de logement. 
          "
        when KEYS[:lessor_denied]
          "
            Après étude de votre candidature sur le logement N°#{@number} nous avons le regret de vous informer que votre dossier n'a pas été retenu par le bailleur.\n
            En effet au vu du nombre de candidatures sur celui-ci certains dossiers ont été retenus comme plus favorables.\n
            Nous vous invitons dès maintenant à vous rendre de nouveau sur votre espace, afin de postuler sur une autre offre de logement.\n
          "
        when KEYS[:cal_denied]
          "
            Après étude de votre candidature sur le logement N°#{@number} nous vous informons que votre dossier n'a pas été retenu.\n
            Nous vous invitons dès maintenant à vous rendre de nouveau sur votre espace, afin de postuler sur une autre offre de logement. 
          "
        when KEYS[:pending], KEYS[:subscribed], KEYS[:applicant_cancelled], KEYS[:applicant_cancelled_after_visit]
          # front only
          ""
        else
          ''
      end
    end

    def actions
      case @key
        when KEYS[:lessor_withdrawal]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:lessor_accepted]
          {
            title: 'Voir ma candidature',
            link: '#connexion-demandeur'
          }
        when KEYS[:lessor_transferred]
          {
            title: 'Voir ma candidature',
            link: '#connexion-demandeur'
          }
        when KEYS[:lessor_cancelled]
          {
            title: 'Voir ma candidature',
            link: '#connexion-demandeur'
          }
        when KEYS[:lessor_denied]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:cal_denied]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:pending], KEYS[:subscribed], KEYS[:applicant_cancelled], KEYS[:applicant_cancelled_after_visit]
          # front only
          {
            title: '',
            link: ''
          }
        else
          ''
      end
    end

    def unsubscribe_link
      ''
    end 
    
  end
end