# frozen_string_literal: true

module Hermes
  # HousingOffer serializer
  class HousingOfferSerializer < Hermes::ApplicationSerializer
    type 'housing_offers'

    attribute :accept_low_occupation
    attribute :address
    attribute :address_can_be_shared
    attribute :address_complements
    attribute :address_extended
    attribute :age
    attribute :agency_id
    attribute :already_inhabited
    attribute :already_rent
    attribute :als_id
    attribute :application_following_contact
    attribute :availability_date
    attribute :bathrooms
    attribute :bedrooms
    attribute :building_type
    attribute :cellars
    attribute :communicate_exact_address
    attribute :condo
    attribute :construction_type
    attribute :construction_year
    attribute :country
    attribute :dalo
    attribute :date_publication_end
    attribute :date_publication_start
    attribute :delai_reponse
    attribute :department
    attribute :description
    attribute :designation_date_end
    attribute :details
    attribute :disability_accesses
    attribute :district
    attribute :dpe_conso
    attribute :dpe_ges
    attribute :duplex
    attribute :effort_rate
    attribute :elevators
    attribute :eligible_count, if: :eligible_count?
    attribute :eligible_requests_nb, if: :eligible_requests_nb?
    attribute :entry_date
    attribute :exclusivity_end_date
    attribute :expiry_days
    attribute :external_ref
    attribute :first_quartile
    attribute :first_quartile
    attribute :floor
    attribute :floors
    attribute :furnished_lodging
    attribute :gardens
    attribute :green_area
    attribute :guarantee_deposit
    attribute :has_cellar
    attribute :has_disability_access
    attribute :has_elevator
    attribute :has_garden
    attribute :has_gardian
    attribute :heating_type
    attribute :hot_water_distribution
    attribute :individual_heating
    attribute :insee_code
    attribute :is_from_form
    attribute :is_public
    attribute :kind
    attribute :kitchen_type
    attribute :latitude
    attribute :lessor_convention_reference
    attribute :lessor_status
    attribute :lessor_status_id
    attribute :lessor_status_updated_at
    attribute :longitude
    attribute :main_picture
    attribute :min_effort_rate
    attribute :mobility
    attribute :mutation
    attribute :new_offer
    attribute :number_of_companies_30_minutes_around
    attribute :offer_status
    attribute :offer_status_id
    attribute :offer_status_updated_at
    attribute :ordered_picture_ids
    attribute :other_quartiles
    attribute :outside
    attribute :parking_charges
    attribute :parking_type
    attribute :picture_urls
    attribute :pictures
    attribute :pictures_nb
    attribute :pmr
    attribute :possible_eligible_requests_nb, if: :possible_eligible_requests_nb?
    attribute :postal_code
    attribute :postal_code
    attribute :potential_request_name_county
    attribute :potential_request_name_town
    attribute :priority_public
    attribute :publication_end_date
    attribute :qpv
    attribute :reference
    attribute :release_date
    attribute :rent_amount
    attribute :rent_with_charges
    attribute :rental_charges
    attribute :requalification
    attribute :reservation_agreement_ref
    attribute :reserved
    attribute :residence_code
    attribute :residence_title
    attribute :rooms
    attribute :rpls_id
    attribute :single_parent
    attribute :subsidiary_company
    attribute :surface
    attribute :tense_area
    attribute :terraces_or_balconies
    attribute :threshold_subtype
    attribute :threshold_type
    attribute :total_charges
    attribute :total_floors
    attribute :type
    attribute :typology
    attribute :virtual_visit
    attribute :applicated_nb
    attribute :async_eligible_department_nb

    RESTRICTED_ATTRIBUTES = %i[
      accept_low_occupation
      active_reservations
      active_reservations_rex_ids
      administration_area
      administrator
      agency_id
      agency_name
      als_siren
      async_eligible_bordering_nb
      async_eligible_other_from_department_nb
      async_eligible_requests_nb
      async_eligible_seeked_bordering_nb
      async_eligible_seeked_nb
      auto_destroy_next_day
      ceil_category_1
      ceil_category_10
      ceil_category_11
      ceil_category_12
      ceil_category_2
      ceil_category_3
      ceil_category_4
      ceil_category_5
      ceil_category_6
      ceil_category_7
      ceil_category_8
      ceil_category_9
      commercialized
      eligibility_query_base
      eligibility_query_bordering
      eligibility_query_department
      eligibility_query_other_from_department
      eligibility_query_seeked
      eligibility_query_seeked_bordering
      entities_siret
      entity_ids
      floor_category_1
      floor_category_10
      floor_category_11
      floor_category_12
      floor_category_2
      floor_category_3
      floor_category_4
      floor_category_5
      floor_category_6
      floor_category_7
      floor_category_8
      floor_category_9
      hash_xml
      housing_channel_keys
      is_mad
      is_mad_updated_at
      last_send_has_error
      last_successful_xml_sent_date
      last_update_by_lessor
      lessor
      lessor_code
      lessor_contact_name
      lessor_email
      lessor_id
      lessor_name
      lessor_orientations
      lessor_phone_number
      lessor_siren
      mad_als_date
      min_effort_treshold
      min_revenues
      nb_of_eligible_employees
      nb_of_requests_from_employees
      operation_name
      reference_mad
      reservation_holder_id
      reserving_business_name
      reserving_siren
      residence_code
      target_objective_ids
      test_created_at
      threshold_id
    ]

    RESTRICTED_ATTRIBUTES.each do |a|
      attribute a, if: :access_for_restricted_attributes?
    end

    if HERMES_CONFIG['DEBUG_ERRORS'] == false
      attribute :last_xml_validation_report
    end

    def not_public?
      !@instance_options[:is_public]
    end 

    def responsible
      return '' unless not_public? || access_for_restricted_attributes?
      super
    end 

    def access_for_restricted_attributes?
      return false unless not_public? && current_user
      roles = (current_user.auth_roles || []) + (current_user.roles || [])
      roles.any? { |role| role != 'demandeur' }
    end 

    def added_to_object_cache_key
      if eligible_requests_nb? || possible_eligible_requests_nb? || eligible_count? || access_for_restricted_attributes?
        return (DateTime.current.to_i / 10).to_s
      end
      "" 
    end 

    def applicated_nb
      return object.applicated_nb unless @instance_options[:recount_applicated_nb]
      object.find_housing_application_with_entity_id(@instance_options[:entities_used_for_eligible_requests_nb]).count
    end 
    
    def lessor_id
      object.lessor_id&.to_s
    end
    
    def entity_ids
      object.entity_ids&.map(&:to_s)
    end
    
    def target_objective_ids
      object.target_objective_ids&.map(&:to_s)
    end
    
    def agency_id
      object.agency_id&.to_s
    end
    
    # def agency_name
    #   object.agency&.title
    # end
    
    def threshold_id
      object.threshold_id&.to_s
    end
    
    def offer_status_id
      object.offer_status_id.to_s
    end
    
    def lessor_status_id
      object.lessor_status_id.to_s
    end
    
    def lessor
      if object.lessor
        Hermes::LessorSerializer.new(
          object.lessor, scope:
          scope, root: false,
          reduced: true)
      end
    end
    
    # pictures
    def ordered_picture_ids
      object.ordered_picture_ids&.map(&:to_s) || []
    end
    
    def default_picture
      {
        "is_default" => true,
        "thumb240" => google_map_link(240),
        "thumb1000" => google_map_link(1000),
        "full_size" => google_map_link(1500),
        "housing_offer_id" => object.id
      }
    end
    
    def google_map_link(size)
      require 'cgi'
      key = (HERMES_CONFIG['google_street_view'] && HERMES_CONFIG['google_street_view']['key']) || "AIzaSyB44UtIXYeMCKhtwUWeUdO16KVDMUAHHqQ"
      signature = (HERMES_CONFIG['google_street_view'] && HERMES_CONFIG['google_street_view']['signature']) || "svUpEVUsO9VuYG24KbDi89Gndw0="
      base_url = (HERMES_CONFIG['google_street_view'] && HERMES_CONFIG['google_street_view']['cache_url']) || "https://maps.googleapis.com"
      location = CGI.escape("#{object.address} #{object.postal_code} #{object.city}")

      url = "#{base_url}/maps/api/streetview?size=#{size}x#{size}&location=#{location}&key=#{key}"
      GoogleUrlSigner.sign(url, signature)
    end
    
    def main_picture
      return default_picture unless main = object.main_picture
      main.is_a?(Hermes::Picture) ? Hermes::PictureSerializer.new(main, scope: scope, root: false) : format_picture_url(main)
    end
    
    def format_picture_url(picture_url)
      {
        "is_picture_url"=> true,
        "thumb240"=> picture_url["thumb240"]||picture_url["thumb1000"]||picture_url["full_size"],
        "thumb1000"=> picture_url["thumb1000"]||picture_url["full_size"]||picture_url["thumb240"],
        "full_size"=> picture_url["full_size"]||picture_url["thumb1000"]||picture_url["thumb240"],
        "housing_offer_id"=> object.id
      }
    end
    
    def pictures
      if object.ordered_pictures && !object.ordered_pictures.empty?
        object.ordered_pictures.map do |picture|
          next unless picture
          Hermes::PictureSerializer.new(picture, scope: scope, root: false)
        end
      elsif object.picture_urls && !object.picture_urls.empty?
        object.picture_urls.map do |picture|
          next unless picture
         format_picture_url(picture)
        end
      else
        return [default_picture]
      end
    end
    
    def pictures_nb
      pictures_nb = 0
      pictures_nb += object.ordered_picture_ids.length if object.ordered_picture_ids 
      pictures_nb += object.picture_urls.length if object.picture_urls
      pictures_nb
    end 
    

    def eligible_count?
      @instance_options[:eligible_count] == true
    end

    def possible_eligible_requests_nb?
      @instance_options[:possible_eligible_requests_nb] == true
    end

    def eligible_count
      object.get_all_eligible_count(true)
    end 

    def eligible_requests_nb?
      @instance_options[:eligible_requests_nb] == true
    end

    def eligible_requests_nb
      return  Hermes::HousingOffer.calculate_estimate(object[:async_eligible_requests_nb]) if object[:async_eligible_requests_nb] && object[:async_eligible_requests_nb] > Hermes::HousingOffer.eligible_request_estimate_min
      requests = Hermes::HousingRequest.where(object.all_eligibility_query)
      return requests&.count(hint: 'all_eligibility') unless @instance_options[:entities_used_for_eligible_requests_nb]
      requests&.where("$and": [ {"entity_id": { "$in": @instance_options[:entities_used_for_eligible_requests_nb]}}])&.count(hint: 'all_eligibility')
    end 

    def possible_eligible_requests_nb
      query = object.all_eligibility_query
      query[:request_status] = {'$ne': 'SATISFAITE' }
      requests = Hermes::HousingRequest.where(query)
      return requests&.count(hint: 'all_eligibility') unless @instance_options[:entities_used_for_eligible_requests_nb]
      requests&.where("$and": [ {"entity_id": { "$in": @instance_options[:entities_used_for_eligible_requests_nb]}}])&.count(hint: 'all_eligibility')
    end 


  end
end
