# frozen_string_literal: true

module Hermes
  # Lessor serializer
  class LessorSerializer < Hermes::ApplicationSerializer
    attributes %i[
      als_id
      title
      siret
      siren
      is_root
      root_id
      parent_id
      id_rex
      id_cristal
      api_id
      rpls_id
      affiliate
      housing_actual_flow_nb
      housing_targeted_flow_nb
      new_housing_actual_flow_nb
      new_housing_targeted_flow_nb
      agency_ids
      lessor_objective_ids
      group_id
      supervisor_group_id
      manager_group_id
      current_user_right
      manual_mode
    ]

    def added_to_object_cache_key
      current_user&.id
    end

    type 'lessors'
    def manual_mode
      object&.manual_mode if object.respond_to? :manual_mode
    end

    def agency_ids
      object.agency_ids.map(&:to_s)
    end

    def lessor_objective_ids
      return if @instance_options[:reduced]
      object.lessor_objective_ids.map(&:to_s)
    end

    def housing_actual_flow_nb
      object.housing_actual_flow_number
    end

    def housing_targeted_flow_nb
      object.housing_targeted_flow_number
    end

    def new_housing_actual_flow_nb
      object.new_housing_actual_flow_number
    end

    def new_housing_targeted_flow_nb
      object.new_housing_targeted_flow_number
    end

    def group_id
      object.group_id&.to_s
    end

    def supervisor_group_id
      object.supervisor_group_id
    end

    def manager_group_id
      object.manager_group_id
    end

    def current_user_right
      return unless current_user
      right = Hermes::UserRight.find_by(user_id: current_user.id, lessor_id: id)
      return if !right
      {
        id: id,
        right: right.right,
        type: right.type,
        title: right.title,
        group_id: right.group_id,
        entity_id: right.entity_id,
        lessor_id: right.lessor_id,
        user_id: right.user_id,
      } 
    end
    
    # has_one :image
  end
end
