module BatchTasks
  class UpdateSeniorityPoints < Base
    
    def update_housing_requests(tagset_id: nil)
      starttime = Time.now
      @log_name = "#{print_time(starttime)}_update_seniority_points.csv"

      ids = Hermes::HousingRequest.where(
        is_active: true, 
        :request_status.nin => ['SATISFAITE', 'ANNULEE'],
        :$or => [
          {:next_seniority_updated_date.lte => Date.today},
          {next_seniority_updated_date: nil}
        ]
      ).order_by(created_at: :desc).pluck(:id)

      count = ids.length
      continue_paginate = true
      page = 1
      per_page = 10_000
      
      headers = %w[
        id
        als_id
        created_at
        sne_created_at
        active_application_id
        old_next_seniority_updated_date
        next_seniority_updated_date
        old_profile_score
        profile_score
        old_seniority_points
        seniority_points
      ]

      add_to_log_file(headers)

      while continue_paginate  do
        start_index = (page - 1) * per_page
        page_ids = ids[start_index, per_page]
        
        r = Hermes::HousingRequest.where(:id.in=> page_ids).only(
          :sne_created_at,
          :created_at,
          :profile_score,
          :score_details,
          :updated_at,
          :active_application_id,
          :next_seniority_updated_date,
          :als_id
        ).order_by(id: :asc)
      
        r.no_timeout.each_with_index do |hr, i|
          begin 
            old_profile_score = hr.profile_score
            old_seniority_points = hr.score_details['seniority_points']
            old_next_seniority_updated_date = hr.next_seniority_updated_date

            hr.score_details['seniority_points'] = hr.calculate_seniority_points
            hr.profile_score = hr.score_details.values.sum
            hr.update_next_seniority_updated_date

            hr.set(
              updated_at: DateTime.current,
              profile_score: hr.profile_score,
              score_details: hr.score_details,
              next_seniority_updated_date: hr.next_seniority_updated_date,
            )
            hr.update_application_score

            line = [
              hr.id.to_s,
              hr.als_id,
              hr.created_at,
              hr.sne_created_at,
              hr.active_application_id,
              old_next_seniority_updated_date,
              hr.next_seniority_updated_date,
              old_profile_score,
              hr.profile_score,
              old_seniority_points,
              hr.score_details['seniority_points'],
            ]
            add_to_log_file(line)
          rescue => e
            add_to_log_file([hr.id, e.message])
          end 
        end
      
        continue_paginate = count >= start_index + per_page
        page += 1
      end 

      create_log_doc(tagset_id)
    end 
  end
end