require 'rails_helper'

RSpec.describe Hermes::BorderingTown, type: :model do
  describe 'import_from_csv' do
    let!(:bt_updated) {create(:bordering_town, insee_code: '01001', name: 'Test1', bordering_towns: [])}
    let!(:bt_not_updated) {create(:bordering_town, insee_code: '01014', name: 'Arbent', bordering_towns: ['39579', '01152', '01283', '01148'])}
    let!(:bt_multiple_insee) {create(:bordering_town, insee_code: '01025', name: 'Test2', bordering_towns: [])}
    let!(:hr_updated) {create(:complete_request, seeked_district_insee_codes: [bt_updated.insee_code])}
    let!(:hr_not_updated) {create(:complete_request, seeked_district_insee_codes: ["99999"])}

    it 'works' do
      path = [Hermes::Engine.root, %w[spec xml_bouchon bordering_towns.csv]].flatten.join('/').to_s

      Hermes::BorderingTown.import_from_csv(File.read(path))

      expect(bt_updated.reload.name).to eq("L'Abergement-Clémenciat")  
      expect(bt_updated.bordering_towns.length).to eq(6)  
      expect(bt_updated.last_update_from_csv_at).not_to be_nil  
      
      expect(bt_not_updated.reload.name).to eq("Arbent")  
      expect(bt_not_updated.bordering_towns.length).to eq(4)  
      expect(bt_not_updated.last_update_from_csv_at).not_to be_nil  
      
      expect(bt_multiple_insee.reload.name).to eq('Bâgé-la-Ville')  
      expect(bt_multiple_insee.reload.bordering_towns.length).to eq(14)
      expect(bt_multiple_insee.last_update_from_csv_at).not_to be_nil  
      
      bt_new_insee = Hermes::BorderingTown.find_by(insee_code: '01026')
      expect(bt_new_insee.reload.name).to eq('Bâgé-le-Châtel')  
      expect(bt_new_insee.reload.bordering_towns.length).to eq(2)
      expect(bt_new_insee.last_update_from_csv_at).not_to be_nil  
      
      expect(hr_updated.reload.bordering_insee_codes).to eq(bt_updated.bordering_towns)
      expect(hr_updated.eligibility_query_bordering).to include(bt_updated.bordering_towns.first)
      expect(hr_updated.eligibility_query_other_from_department).to include(bt_updated.bordering_towns.first)

      expect(hr_not_updated.reload.bordering_insee_codes).to eq([])
    end

    it 'with skip_hr_update_on_bordering_town_import' do 
      path = [Hermes::Engine.root, %w[spec xml_bouchon bordering_towns.csv]].flatten.join('/').to_s

      HERMES_CONFIG['skip_hr_update_on_bordering_town_import'] = true
      Hermes::BorderingTown.import_from_csv(File.read(path))

      expect(bt_updated.reload.name).to eq("L'Abergement-Clémenciat")  
      expect(bt_updated.bordering_towns.length).to eq(6)  
      expect(bt_updated.last_update_from_csv_at).not_to be_nil  

      expect(hr_updated.reload.bordering_insee_codes).to eq([])
      expect(hr_not_updated.reload.bordering_insee_codes).to eq([])
    end 
  end
end
