require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

RSpec.describe BatchTasks::UpdateSeniorityPoints, type: :service do

  subject { described_class.new.update_housing_requests }

  describe 'update_seniority_points' do 
    let!(:hr) {create(:complete_request)}
    let!(:hr2) {create(:complete_request)}

    it 'works' do 
      Timecop.freeze(Date.new(2023, 11, 10)) do
        hr.update(created_at: Date.today - 1.month)
        expect(hr.reload.next_seniority_updated_date).to eq(Date.new(2023, 12, 11))
      end 
      
      Timecop.freeze(Date.new(2024, 1, 1)) do
        hr2.update(created_at: Date.today)
        expect(hr2.reload.next_seniority_updated_date).to eq(Date.new(2024, 2, 2))
        
        hr.set(profile_score: 99999)
        hr2.set(profile_score: 99999)

        subject

        expect(hr.reload.next_seniority_updated_date).to eq(Date.new(2024, 1, 11))
        expect(hr.reload.profile_score).to eq(2)
        expect(hr.score_details['seniority_points']).to eq(2)

        expect(hr2.reload.profile_score).to eq(99999)
      end
    end 

    it 'when no next_seniority_updated_date' do
      hr.set(next_seniority_updated_date: nil)
      subject
      expect(hr.reload.next_seniority_updated_date).not_to be_nil
    end 
  end 
end